from kakeibox_controllers.controllers.controller_get_actions \
    import KakeiboxControllerGet
from kakeibox_core.entry_points.commands import commands


class TransactionCategoryGet(KakeiboxControllerGet):

    def execute(self, input):
        self.input = input
        self.command = commands.get_category()
        return super(TransactionCategoryGet, self).execute()



from kakeibox_controllers.handlers.kakeibox_handler \
    import AbstractKakeiboxHandler


class KakeiboxPresenterHandler(AbstractKakeiboxHandler):

    def handle(self, request):
        if request and request.presenter and request.command_response:
            presenter = request.presenter
            request.presenter_output = presenter.show(
                request.command_response.value)
        return super().handle(request)



from kakeibox_controllers.controllers.controller_list_actions \
    import KakeiboxControllerList
from kakeibox_core.entry_points.commands import commands


class TransactionCategoryList(KakeiboxControllerList):

    def execute(self):
        self.command = commands.list_categories()
        return super(TransactionCategoryList, self).execute()



from kakeibox_controllers.controllers.controller_base \
    import KakeiboxControllerBase
from kakeibox_controllers.handlers.kakeibox_request \
    import KakeiboxRequest
from kakeibox_controllers.handlers.kakeibox_serializer_handler \
    import KakeiboxSerializerHandler
from kakeibox_controllers.handlers.kakeibox_executor_list_periods_handler \
    import KakeiboxExecutorListPeriodsHandler
from kakeibox_controllers.handlers.kakeibox_presenter_handler \
    import KakeiboxPresenterHandler
from kakeibox_presenter_json.presenter.json_presenter import JsonPresenter
from kakeibox_serializer_json.serializer.json_serializer \
    import JsonInputSerializer


class KakeiboxControllerListPeriod(KakeiboxControllerBase):

    def _create_request(self):
        request = KakeiboxRequest()
        request.input = self._input
        request.command = self._command
        request.input_serializer = JsonInputSerializer()
        request.presenter = JsonPresenter()
        self._request = request
        return request

    def _prepare_chain_of_commands(self):
        serialize = KakeiboxSerializerHandler()
        executor = KakeiboxExecutorListPeriodsHandler()
        presenter = KakeiboxPresenterHandler()
        serialize.set_next(executor).set_next(presenter)
        return serialize



from kakeibox_controllers.handlers.kakeibox_handler \
    import AbstractKakeiboxHandler


class KakeiboxExecutorHandler(AbstractKakeiboxHandler):

    def handle(self, request):
        if request and request.command and request.input_serialized:
            request.command_response = request.command.execute(
                request.input_serialized)
        return super().handle(request)



from kakeibox_controllers.controllers.controller_delete_actions \
    import KakeiboxControllerDelete
from kakeibox_core.entry_points.commands import commands


class TransactionCategoryDelete(KakeiboxControllerDelete):

    def execute(self, input):
        self.input = input
        self.command = commands.delete_category()
        return super(TransactionCategoryDelete, self).execute()



from kakeibox_controllers.controllers.controller_list_actions \
    import KakeiboxControllerList
from kakeibox_core.entry_points.commands import commands


class TransactionSubcategoryList(KakeiboxControllerList):

    def execute(self):
        self.command = commands.list_all_subcategories()
        return super(TransactionSubcategoryList, self).execute()



from kakeibox_controllers.controllers.controller_update_actions \
    import KakeiboxControllerUpdate
from kakeibox_core.entry_points.commands import commands


class TransactionCategoryUpdate(KakeiboxControllerUpdate):

    def execute(self, code, transaction_category):

        self.input = {
            'code': code, 'transaction_category': transaction_category}
        self.command = commands.update_category()
        return super(TransactionCategoryUpdate, self).execute()



from kakeibox_controllers.controllers.controller_base \
    import KakeiboxControllerBase
from kakeibox_controllers.handlers.kakeibox_request \
    import KakeiboxRequest
from kakeibox_controllers.handlers.kakeibox_executor_list_handler \
    import KakeiboxExecutorListHandler
from kakeibox_controllers.handlers.kakeibox_presenter_handler \
    import KakeiboxPresenterHandler
from kakeibox_presenter_json.presenter.json_presenter import JsonPresenter


class KakeiboxControllerList(KakeiboxControllerBase):

    def execute(self):
        if not self._command:
            raise Exception("The request command and input are required")

        if not self._request:
            request = self._create_request()
        chain_commands = self._prepare_chain_of_commands()
        chain_commands.handle(request)
        return request.presenter_output

    def _create_request(self):
        request = KakeiboxRequest()
        request.command = self._command
        request.presenter = JsonPresenter()
        self._request = request
        return request

    def _prepare_chain_of_commands(self):
        executor = KakeiboxExecutorListHandler()
        presenter = KakeiboxPresenterHandler()
        executor.set_next(presenter)
        return executor



class KakeiboxHandler(object):
    def set_next_handler(self, kakeibox_handler):
        raise Exception("Method is not implemented")

    def handle(self, request):
        raise Exception("Method is not implemented")


class AbstractKakeiboxHandler(KakeiboxHandler):
    _next_handler= None

    def set_next(self, handler):
        self._next_handler = handler
        return handler

    def handle(self, request):
        if self._next_handler:
            return self._next_handler.handle(request)
        return request



class KakeiboxControllerBase(object):
    def __init__(self):
        self._request = None
        self._input = None
        self._command = None
        self._request = None

    @property
    def command(self):
        return self._command

    @command.setter
    def command(self, command):
        self._command = command

    @property
    def input(self):
        return self._input

    @input.setter
    def input(self, input):
        self._input = input

    @property
    def request(self):
        return self._input

    @request.setter
    def request(self, request):
        self._request = request

    def execute(self):
        if not self._command or not self._input:
            raise Exception("The request command and input are required")

        if not self._request:
            request = self._create_request()
        chain_commands = self._prepare_chain_of_commands()
        chain_commands.handle(request)
        return request.presenter_output

    def _create_request(self):
        raise Exception("Method is not implemented")

    def _prepare_chain_of_commands(self):
        raise Exception("Method is not implemented")



from kakeibox_controllers.controllers.controller_delete_actions \
    import KakeiboxControllerDelete
from kakeibox_core.entry_points.commands import commands


class TransactionSubcategoryListPerCategory(KakeiboxControllerDelete):

    def execute(self, input):
        self.input = input
        self.command = commands.list_per_category()
        return super(TransactionSubcategoryListPerCategory, self).execute()



class KakeiboxRequest(object):
    input_serializer = None
    input = None
    input_serialized = None
    command = None
    command_response = None
    presenter = None
    presenter_output = None



from kakeibox_controllers.controllers.controller_base \
    import KakeiboxControllerBase
from kakeibox_controllers.handlers.kakeibox_request \
    import KakeiboxRequest
from kakeibox_controllers.handlers.kakeibox_executor_handler \
    import KakeiboxExecutorHandler
from kakeibox_controllers.handlers.kakeibox_presenter_handler \
    import KakeiboxPresenterHandler
from kakeibox_presenter_json.presenter.json_presenter import JsonPresenter


class KakeiboxControllerDelete(KakeiboxControllerBase):

    def _create_request(self):
        request = KakeiboxRequest()
        request.command = self._command
        request.presenter = JsonPresenter()
        request.input_serialized = self._input
        self._request = request
        return request

    def _prepare_chain_of_commands(self):
        executor = KakeiboxExecutorHandler()
        presenter = KakeiboxPresenterHandler()
        executor.set_next(presenter)
        return executor



from kakeibox_controllers.handlers.kakeibox_handler \
    import AbstractKakeiboxHandler


class KakeiboxSerializerHandler(AbstractKakeiboxHandler):

    def handle(self, request):
        if request and request.input_serializer and request.input:
            serializer = request.input_serializer
            request.input_serialized = serializer.serialize(request.input)
        return super().handle(request)



from kakeibox_controllers.handlers.kakeibox_handler \
    import AbstractKakeiboxHandler


class KakeiboxExecutorListHandler(AbstractKakeiboxHandler):

    def handle(self, request):
        if request and request.command:
            request.command_response = request.command.execute()
        return super().handle(request)



from kakeibox_controllers.controllers.controller_new_actions \
    import KakeiboxControllerNew
from kakeibox_core.entry_points.commands import commands


class TransactionCategoryNew(KakeiboxControllerNew):

    def execute(self, input):
        self.input = input
        self.command = commands.new_category()
        return super(TransactionCategoryNew, self).execute()



from kakeibox_controllers.controllers.controller_update_actions \
    import KakeiboxControllerUpdate
from kakeibox_core.entry_points.commands import commands


class TransactionSubcategoryUpdate(KakeiboxControllerUpdate):

    def execute(self, code, transaction_subcategory):
        self.input = {
            'code': code, 'transaction_subcategory': transaction_subcategory}
        self.command = commands.update_subcategory()
        return super(TransactionSubcategoryUpdate, self).execute()
